﻿using System;
using System.IO;

using DevExpress.XtraMap;

namespace TestProject
{
    /// <summary>
    /// 커스텀 맵 타일 소스
    /// </summary>
    public class CustomMapTileSource : MapTileSourceBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 타일 크기
        /// </summary>
        public new const int TileSize = 256;

        /// <summary>
        /// 최대 확대/축소 레벨
        /// </summary>
        public const int MaximumZoomLevel = 2;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 디렉토리 경로
        /// </summary>
        private string directoryPath;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomMapTileSource(cacheOptionsProvider)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="cacheOptionsProvider">캐시 옵션 공급자</param>
        public CustomMapTileSource(ICacheOptionsProvider cacheOptionsProvider) : base
        (
            (int)CalculateTotalImageSize(MaximumZoomLevel),
            (int)CalculateTotalImageSize(MaximumZoomLevel),
            TileSize,
            TileSize,
            cacheOptionsProvider
        )
        {
            DirectoryInfo directoryInfo = new DirectoryInfo(Directory.GetCurrentDirectory());

            directoryPath = directoryInfo.Parent.Parent.FullName;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 전체 이미지 크기 계산하기 - CalculateTotalImageSize(zoomLevel)

        /// <summary>
        /// 전체 이미지 크기 계산하기
        /// </summary>
        /// <param name="zoomLevel">확대/축소 레벨</param>
        /// <returns>전체 이미지 크기</returns>
        public static double CalculateTotalImageSize(double zoomLevel)
        {
            if(zoomLevel < 1.0)
            {
                return zoomLevel * TileSize * 2;
            }

            return Math.Pow(2.0, zoomLevel) * TileSize;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 확대/축소 레벨에 따라 타일 구하기 - GetTileByZoomLevel(zoomLevel, tilePositionX, tilePositionY)

        /// <summary>
        /// 확대/축소 레벨에 따라 타일 구하기
        /// </summary>
        /// <param name="zoomLevel">확대/축소 레벨</param>
        /// <param name="tilePositionX">타일 위치 X</param>
        /// <param name="tilePositionY">타일 위치 Y</param>
        /// <returns>타일 URI</returns>
        public override Uri GetTileByZoomLevel(int zoomLevel, int tilePositionX, int tilePositionY)
        {
            if(zoomLevel <= MaximumZoomLevel)
            {
                Uri uri = new Uri
                (
                    string.Format
                    (
                        "file://" + directoryPath + "\\openstreetmap.org\\Hybrid_{0}_{1}_{2}.png",
                        zoomLevel,
                        tilePositionX,
                        tilePositionY
                    )
                );

                return uri;
            }

            return null;
        }

        #endregion
    }
}